# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-04-10 06:14
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Alternative',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('content', models.TextField()),
                ('mimetype', models.CharField(max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filename', models.CharField(blank=True, default=None, max_length=255, null=True)),
                ('content', models.BinaryField(blank=True, default=None, null=True)),
                ('mimetype', models.CharField(blank=True, default=None, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('m_from', models.TextField(verbose_name='From')),
                ('subject', models.CharField(max_length=78)),
                ('body', models.TextField()),
                ('created', models.DateTimeField(auto_now_add=True)),
            ],
        ),
        migrations.CreateModel(
            name='Header',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255)),
                ('value', models.CharField(max_length=255)),
                ('email', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='headers', to='mcfeely.Email')),
            ],
        ),
        migrations.CreateModel(
            name='Queue',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('queue', models.CharField(max_length=50)),
                ('description', models.CharField(max_length=200, verbose_name='Queue Description')),
                ('display_to_user', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Recipient',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('address', models.CharField(max_length=254)),
                ('recipient_type', models.CharField(choices=[('to', 'To'), ('cc', 'CC'), ('bcc', 'BCC')], max_length=3)),
                ('status', models.CharField(choices=[('in_queue', 'In Queue'), ('sent_success', 'Sent Successfuly'), ('deferred', 'Deferred'), ('blocked_unsubscribe', 'Blocked by unsubscribe'), ('failure', 'Failure')], default='in_queue', max_length=100)),
                ('email', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mcfeely.Email')),
            ],
        ),
        migrations.CreateModel(
            name='Unsubscribe',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('address', models.EmailField(max_length=254, verbose_name='Email Address')),
                ('added', models.DateTimeField(auto_now_add=True)),
                ('queue', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='mcfeely.Queue')),
            ],
            options={
                'verbose_name': 'Usubscribe',
                'verbose_name_plural': 'Unsubscribe',
            },
        ),
        migrations.AddField(
            model_name='email',
            name='queue',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mcfeely.Queue'),
        ),
        migrations.AddField(
            model_name='attachment',
            name='email',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='mcfeely.Email'),
        ),
        migrations.AddField(
            model_name='alternative',
            name='email',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='alternatives', to='mcfeely.Email'),
        ),
        migrations.AlterUniqueTogether(
            name='unsubscribe',
            unique_together=set([('address', 'queue')]),
        ),
    ]



import base64
from django.db import models
try:
    from django.utils.encoding import force_bytes
except:
    pass
from django.utils.encoding import smart_bytes, smart_text

class Base64Field(models.TextField):

    def to_python(self, value):
        if value is None:
            return None
        return base64.b64decode(smart_bytes(value))

    def get_prep_value(self, value):
        if value is None:
            return None
        return smart_text(base64.b64encode(smart_bytes(value)))




from django.core.management.base import BaseCommand
from django.core.mail import get_connection
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.conf import settings
from django.db.models import Q

from socket import error as socket_error
import smtplib
from optparse import make_option

from mcfeely.models import Email
from mcfeely.models import Queue
from mcfeely.models import Unsubscribe
from mcfeely.models import Alternative
from mcfeely.models import Attachment
from mcfeely.models import Header
from mcfeely.models import Recipient

EMAIL_BACKEND = getattr(
    settings,
    'MCFEELY_EMAIL_BACKEND',
    'django.core.mail.backends.smtp.EmailBackend'
)

class Command(BaseCommand):
    args = '<queue_name>'
    option_list = BaseCommand.option_list + (
        make_option('--retry',
            action='store_true',
            dest='retry',
            default=False,
            help='retry Deferred email'),
        )
    help = 'send all mail in the specified queue'

    def __messages(self, queue, status):
        messages = Email.objects.filter(
            queue=queue,
            recipient__status=status).distinct()

        for message in messages:
            for recipient in message.recipient_set.all():
                if self.__check_unsubscribe(recipient.address):
                    recipient.status = 'blocked_unsubscribe'
                    recipient.save()

        return(messages)

    def __check_unsubscribe(self, address):
        try:
            Unsubscribe.objects.get(address=address)
            return True
        except Unsubscribe.DoesNotExist:
            return False


    def handle(self, *args, **options):
        connection = get_connection(backend=EMAIL_BACKEND)

        if options['retry']:
            status='deferred'
        else:
            status='in_queue'

        if len(args) == 0:
            args = [x['queue'] for x in Queue.objects.all().values()]

        for queue_name in args:
            queue = Queue.objects.get(queue=queue_name)

            messages = self.__messages(queue, status=status)
            if messages:
                for message in messages:
                    alternatives = Alternative.objects.filter(
                        email=message).values_list('content', 'mimetype')
                    attachments = Attachment.objects.filter(email=message)

                    headers = Header.objects.filter(
                        email=message).values('key', 'value')
                    mail_headers = {}
                    for header in headers:
                        mail_headers[header['key']] = header['value']

                    recipients = message.recipient_set.filter(
                        status=status)
                    message_to = recipients.filter(
                        recipient_type='to',
                        ).values_list('address', flat=True)
                    message_bcc = message.recipient_set.filter(
                        recipient_type='bcc',
                        ).values_list('address', flat=True)
                    message_cc = message.recipient_set.filter(
                        recipient_type='cc',
                        ).values_list('address', flat=True)

                    if alternatives:
                        email = EmailMultiAlternatives(
                            message.subject,
                            message.body,
                            message.m_from,
                            message_to,
                            message_bcc,
                            connection,
                            None,
                            headers=mail_headers,
                            alternatives=alternatives,
                            cc=message_cc,
                        )
                    else:
                        email = EmailMessage(
                            message.subject,
                            message.body,
                            message.m_from,
                            message_to,
                            message_bcc,
                            connection,
                            None,
                            headers=mail_headers,
                            cc=message_cc,
                        )

                    if attachments:
                        for attachment in attachments:
                            email.attach(
                                attachment.filename,
                                attachment.content,
                                attachment.mimetype
                            )

                    try:
                        email.send()                        
                        for recipient in recipients:
                                recipient.status = 'sent_success'
                                recipient.save()

                    except (socket_error, smtplib.SMTPSenderRefused,
                            smtplib.SMTPRecipientsRefused,
                            smtplib.SMTPAuthenticationError):
                        for recipient in recipients:
                            recipient.status = 'deferred'
                            recipient.save()


from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.conf import settings
from mcfeely.models import Queue

def default_queue(queue):
    if queue is None:
        default_queue = getattr(
            settings, 'DEFAULT_EMAIL_QUEUE', 'Default'
        )
        return(Queue.objects.get(queue=default_queue))
    if isinstance(queue, str):
        return(Queue.objects.get(queue=queue))
    else:
        return(queue)


class QueueEmailMessage(EmailMessage):
    """ Override EmailMessage to allow the addition of queues """
    def __init__(self, subject='', body='', from_email=None, to=None,
                 bcc=None, connection=None, attachments=None, headers=None,
                 cc=None, queue=None):

        super(QueueEmailMessage, self).__init__(
            subject,
            body,
            from_email,
            to,
            bcc,
            connection,
            attachments,
            headers,
            cc)
        self.queue = default_queue(queue)


class QueueEmailMultiAlternatives(EmailMultiAlternatives):
    """ Override EmailMultiAlternatives to allow the addition of
    queues """
    def __init__(self, subject='', body='', from_email=None, to=None, bcc=None,
                 connection=None, attachments=None, headers=None,
                 alternatives=None, cc=None, queue=None):
        super(QueueEmailMultiAlternatives, self).__init__(
            subject,
            body,
            from_email,
            to,
            bcc,
            connection,
            attachments,
            headers,
            cc)
        self.queue = default_queue(queue)



from django.db import models
from mcfeely.fields import Base64Field

STATUS = (
    ('in_queue', 'In Queue'),
    ('sent_success', 'Sent Successfuly'),
    ('deferred', 'Deferred'),
    ('blocked_unsubscribe', 'Blocked by unsubscribe'),
    ('failure', 'Failure'),
)
RECIPIENT_TYPE = (
    ('to', 'To'),
    ('cc', 'CC'),
    ('bcc', 'BCC'),
)


class Queue(models.Model):
    queue = models.CharField(max_length=50)
    description = models.CharField('Queue Description', max_length=200)
    display_to_user = models.BooleanField(default=False)

    def __unicode__(self):
        return(self.description)

    def __str__(self):
        return(self.description)


class Email(models.Model):

    """ Queued up Emails to be sent out. """
    m_from = models.TextField('From')

    # Subjects can be be longer if they're split byt CRLF, but this is a good
    # default
    subject = models.CharField(max_length=78)
    body = models.TextField()
    queue = models.ForeignKey(Queue)
    created = models.DateTimeField(auto_now_add=True)

    def __unicode__(self):
        return('[ %s ] - %s' % (self.queue, self.subject))


class Recipient(models.Model):
    email = models.ForeignKey(Email)
    address = models.CharField(max_length=254)
    recipient_type = models.CharField(max_length=3, choices=RECIPIENT_TYPE)
    status = models.CharField(
        max_length=100, default='in_queue', choices=STATUS)

    def __unicode__(self):
        return('%s : %s' % (self.recipient_type, self.address))


class Alternative(models.Model):
    email = models.ForeignKey(Email, related_name='alternatives')
    content = models.TextField()
    mimetype = models.CharField(max_length=255)


class Attachment(models.Model):
    email = models.ForeignKey(Email, related_name='attachments')
    filename = models.CharField(
        max_length=255, null=True, blank=True, default=None)
    content = Base64Field(null=True, blank=True, default=None)
    mimetype = models.CharField(
        max_length=255, null=True, blank=True, default=None)


class Header(models.Model):
    email = models.ForeignKey(Email, related_name='headers')
    key = models.CharField(max_length=255)
    value = models.CharField(max_length=255)


class Unsubscribe(models.Model):

    """ Any email addresses in this table we will not send mail to.  """
    address = models.EmailField('Email Address')
    added = models.DateTimeField(auto_now_add=True)
    queue = models.ForeignKey(Queue, blank=True, null=True)

    class Meta:
        verbose_name = ('Usubscribe')
        verbose_name_plural = ('Unsubscribe')
        unique_together = ('address', 'queue')

    def __unicode__(self):
        return(self.address)
